/* ram.c - 6502 microcomputer emulator
 *
 * Copyright (C) 2017 Timothy Richard Musson
 *
 * Email: Tim Musson <trmusson@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include "xlib/xutil.h"
#include "xlib/xmem.h"

#include "config.h"
#include "profile.h"
#include "memory.h"
#include "ram.h"


struct _RAM {

	/* Memory */

	MemoryType     type;

	int            base;
	int            size;
	xbool_t         read_only;

	MemoryFreeData_f  free_data;
	MemoryStoreByte_f store_byte;
	MemoryReadByte_f  read_byte;

	Byte           *data;

	/* RAM */

	/* No special features */

};


RAM*
ram_new (Profile *p, int ram_size)
{
	RAM *r;

	r = xnew0 (RAM, 1);

	mem_init ((Memory*)r, p->ram_base,
	    ram_size == -1 ? p->ram_size : ram_size, !p->ram_clear, MEM_TYPE_RAM);

	return r;
}


void
ram_free (RAM *r)
{
	if (r == NULL) {
		return;
	}

	r->free_data ((Memory*)r);
	xfree (r);
}

