/* xlist.h
 *
 * Copyright (C) 2005, 2014, 2020 Timothy Richard Musson
 *
 * Email: Tim Musson <trmusson@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */

#ifndef XLIST_H
#define XLIST_H


#define XLIST_POOL_GROW_SIZE 1024

typedef void (*XList_f)(const void *data, const void *user_data);

typedef struct _xlist xlist_t;

struct _xlist {
	void    *data;
	xlist_t *next;
};


#define xlist_is_empty(list)     ((list) == NULL)
#define xlist_head(list)         (list)
#define xlist_tail(list)         ((list)->next)
#define xlist_next(list)         ((list)->next)
/* #define xlist_nth_data(list, n)  xlist_nth((list), (n))->data */


int      xlist_pool_n_taken (void);

void     xlist_pool_init (size_t growsz);
void     xlist_pool_free (void);
int      xlist_pool_n_taken (void);

xlist_t* xlist_node_get (void *data);
void     xlist_node_put (xlist_t *node);

void     xlist_free (xlist_t **list);

void*    xlist_pop (xlist_t **list);
void     xlist_push (xlist_t **list, void *data);
void     xlist_append (xlist_t **list, void *data);
void     xlist_move (xlist_t **dest, xlist_t **src);
void     xlist_join (xlist_t **dest, xlist_t **src);
xlist_t* xlist_copy (xlist_t *list);
int      xlist_length (xlist_t *list);

xlist_t* xlist_unlink (xlist_t **list, xlist_t *node);
void*    xlist_remove (xlist_t **list, xlist_t *node);
void     xlist_uniq (xlist_t *list, XCmp_f cmp, XList_f freefunc);

void     xlist_insert_nth (xlist_t **list, int n, void *data);
void     xlist_insert_before (xlist_t **list, xlist_t *node, void *data);
void     xlist_insert_sorted (xlist_t **list, xlist_t *node, XCmp_f cmp);
void     xlist_insert_sort (xlist_t **list, XCmp_f cmp);

xlist_t* xlist_nth (xlist_t *list, int n);
void*    xlist_nth_data (xlist_t *list, int n);
xlist_t* xlist_last (xlist_t *list);
xlist_t* xlist_find (xlist_t *list, const void *data);
xlist_t* xlist_find_cmp (xlist_t *list, const void *data, XCmp_f cmp);
int      xlist_index (xlist_t *list, const void *data);
int      xlist_index_cmp (xlist_t *list, const void *data, XCmp_f cmp);
int      xlist_index_node (xlist_t *list, xlist_t *node);

void     xlist_swap_data (xlist_t *list, int a, int b);
void     xlist_shuffle_data (xlist_t **list);
void     xlist_reverse (xlist_t **list);
void     xlist_foreach (xlist_t *list, XList_f func, void *data);


#endif /* XLIST_H */

