/* xstring.h
 *
 * Copyright (C) 2005, 2012, 2020 Timothy Richard Musson
 *
 * Email: Tim Musson <trmusson@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */

#ifndef XSTRING_H
#define XSTRING_H


/* Default amount to increase allocated space as strings grow. */
#define XSTRING_GROW_SIZE   255

typedef struct _xstring xstring_t;

struct _xstring {
	char *str;
	size_t size;
	size_t len;
	size_t growsz;
};

#define XSTRING_SIZE(xs)  ((xs)->size)
#define XSTRING_LEN(xs)   ((xs)->len)
#define XSTRING_STR(xs)   ((xs)->str)

xstring_t*   xstring_alloc (size_t size, size_t growsz);
xstring_t*   xstring_new (const char *str);
char*        xstring_free (xstring_t *xs, xbool_t free_str);
xstring_t*   xstring_append_c (xstring_t *xs, int c);
xstring_t*   xstring_append (xstring_t *xs, const char *str);
xstring_t*   xstring_assign (xstring_t *xs, const char *str);
xstring_t*   xstring_truncate (xstring_t *xs, size_t len);
int          xstring_cmp (xstring_t *x1, xstring_t *x2);
int          xstring_ncmp (xstring_t *x1, xstring_t *x2, size_t n);

#if 0

/* TODO: */

xstring_t*   xstring_insert_c (xstring_t *xs, size_t pos, int c);
xstring_t*   xstring_insert (xstring_t *xs, size_t pos, const char *str);
xstring_t*   xstring_prepend_c (xstring_t *xs, int c);
xstring_t*   xstring_prepend (xstring_t *xs, const char *str);
xstring_t*   xstring_cut (xstring_t *xs, size_t pos, size_t len);

#endif


#endif /* XSTRING_H */

